defmodule LinkShortener.Links.Links do
  use Ecto.Schema
  import Ecto.Changeset
  import Ecto.Query
  alias LinkShortener.Repo
  alias LinkShortener.Links.Link
  alias LinkShortener.Generators.SafeString

  def new_one(), do: Link.changeset(%Link{})

  def create_one(attrs, length \\ 10, generator \\ SafeString) do
    shorten = generator.generate(length)
    insert_one(Map.put(attrs, :shorten, shorten))
  end

  def insert_one(attrs) do
    %Link{}
    |> Link.changeset(attrs)
    |> Repo.insert()
  end

  def edit_one(id) do
    get_one(id)
    |> Link.changeset()
  end

  def update_one(%Link{} = link, changes) do
    Link
    |> Link.changeset(changes)
    |> Repo.update()
  end

  def insert_one(attrs) do
    %Link{}
    |> Link.changeset(attrs)
    |> Repo.insert()
  end

  def delete_one(%Link{} = link), do: Repo.delete(link)

  def get_one_by(attrs) do
    Repo.get_by(Link, attrs)
  end

  def get_one_by_shorten(shorten) do
    get_one_by(%{shorten: shorten})
  end

  def get_one(id) do
    Repo.get!(Link, id)
  end

  def get_all(opts) do
    from(Link)
    |> Repo.all()
  end
end
